/*
 *  KHLAEvent.h
 *  KDataStructure
 *
 *  Created by Adam Cox on 2/26/10.
 *  Copyright 2010 Karlsruhe Institute of Technology. All rights reserved.
 *
 */
#ifndef __KHLAEVENT_H__
#define __KHLAEVENT_H__

#include "KEvent.h"

//system record forward declarations
#include "TRef.h"
//sub record forward declarrations
class TClonesArray; 
class KSambaSubRecord;
class KSingleBoloSubRecord;

class TBits;

const Char_t kHLAEventName[] = "HLA";
const Char_t kHLAClassName[] = "KHLAEvent";

class KHLAEvent : public KEvent {         
	
public:
	//Constructors and destructor. Only Derived class and the Factory
	//SHOULD access these methods. 
	//However, in order for ROOT's I/O to work properly
	//the constructor and destructor
	//have to be public. Otherwise, I would have made them protected or private
	KHLAEvent(void);
	KHLAEvent(const KHLAEvent &anEvent);
	virtual ~KHLAEvent(void);
	KHLAEvent& operator=(const KEvent &anEvent);
	KHLAEvent& operator=(const KHLAEvent &anEvent);
	virtual void Clear(Option_t *option = "C");
	virtual void ClearArrays(Option_t *anOption = "C");
	Bool_t IsSame(const KHLAEvent &anEvent, Bool_t bPrint = false) const;
	Bool_t operator==(const KHLAEvent &anEvent) const { return IsSame(anEvent,false); }
	Bool_t operator!=(const KHLAEvent &anEvent) const { return !(*this==anEvent); }
	virtual void Compact(void);
	

	//Getters and Setters to member variables
	
	Int_t GetNumSambas(void) const {return fNumSamba;}
	Int_t GetNumBolos(void) const {return fNumBolo;}
	
	
	KSambaSubRecord* GetSamba(Int_t i) const;
	KSingleBoloSubRecord* GetBolo(Int_t i) const;
	
	TClonesArray* GetSambaSubRecords(void) const {return fSamba;}
	TClonesArray* GetBoloSubRecords(void) const {return fBolo;}
	
	//Adders
	KSambaSubRecord* AddSamba();
	KSingleBoloSubRecord* AddBolo();
	
		
		
	void SetAssignmentOptionNoTref(Bool_t anOpt){fAssignmentOptionNoTref = anOpt;}
	Bool_t GetAssignmentOptionNoTref(void){ return fAssignmentOptionNoTref;}
	
	//Only use these if you know what you're doing. 
	void SetNumSamba(Int_t aNum){fNumSamba = aNum;} //Only use these if you know what you're doing. 
	void SetNumBolo(Int_t aNum){fNumBolo = aNum;} //Only use these if you know what you're doing. 
	
	//Print data members //Probably it would be much easier to implement it in Event and just overload it, but I don't really know how to do it and I only really nead it for testing.
	
private: 
	Bool_t fAssignmentOptionNoTref;
	
	//Array of SubRecords containing event information
	Int_t fNumSamba; //the number of Samba Sub Records in this event
	Int_t fNumBolo; //the number of Single Bolo Sub Records in this event
	
	TClonesArray *fSamba; //->  pointer to the array of Samba Sub Records
	TClonesArray *fBolo; //->  pointer to the array of Single Bolo Sub Records

	//private methods
	template<class T> T* AddSubRecord(TClonesArray *mArray, Int_t &mCount);
	void ClearArray(Option_t *anOpt, TClonesArray *mArray, Int_t &mCount);
	void CreateArrays(void);
	void InitializeMembers(void);
	void CopyLocalMembers(const KHLAEvent &anEvent);
	void CopyClonesArrays(const KHLAEvent &anEvent);
	
	ClassDef(KHLAEvent ,1);
};


#endif // __KHLAEVENT_H__
