//_____________________________________________
//
// KSambaSubRecord.cxx
// KDataStructure
//
// Author: Adam Cox <mailto:adam.cox@ik.fzk.de> on 3/25/10.
//
// * Copyright 2010 Karlsruhe Institute of Technology. All rights reserved.
//
// The base class for all Samba Sub Records. This contains event information
// that is from the particular samba machine that the data was acquired on.
// 
//

#include "KSambaSubRecord.h"
#include <stdio.h>
#include <iostream>
#include <cstring>

ClassImp(KSambaSubRecord);

KSambaSubRecord::KSambaSubRecord(void)
{
	InitializeMembers();
	
	//We must deal with members on the heap separately 
	//from members on the stack
	
}


KSambaSubRecord::KSambaSubRecord(const KSambaSubRecord &aRec)
: KSubRecord(aRec)
{
	CopyLocalMembers(aRec);
	
}

KSambaSubRecord& KSambaSubRecord::operator=(const KSambaSubRecord &aRec)
{
	if(&aRec == this) {
		return *this;
		cout << "KSambaSubRecord:operator= Woops - copying to yourself" << endl;
	}
	
	this->KSubRecord::operator=(aRec);
	CopyLocalMembers(aRec);
	
	return *this;
}

void KSambaSubRecord::CopyLocalMembers(const KSambaSubRecord &aRec)
{
	SetSambaEventNumber(aRec.GetSambaEventNumber()); 
	SetSambaDAQNumber(aRec.GetSambaDAQNumber());
	
}

KSambaSubRecord::~KSambaSubRecord(void)
{
	Clear("C");

	
}

void KSambaSubRecord::Clear(Option_t *opt)
{
	//Clear the base classes and then clear/delete any local
  //members. Its necessary for this Clear method to exist
  //in the case that instances of this object are stored
  //inside of a TClonesArray
  //Also, if this class holds any TClonesArrays, it must call
  //TClonesArray::Clear("C")

	KSubRecord::Clear(opt);
	
  //Clear and delete local objects here. 
	//delete local stuff here


  //Re initialize local members here and prepare for the next use of this class.
  InitializeMembers();

}


void KSambaSubRecord::InitializeMembers(void)
{
	//Init local members
	
	//WARNING - THIS METHOD SHOULD NEVER ALLOCATE SPACE FOR POINTERS
  //ONLY SET MEMBERS ON THE STACK TO THEIR INITIAL VALUES
	
	SetSambaEventNumber(-99); 
	SetSambaDAQNumber(-99);

}


Bool_t KSambaSubRecord::IsSame(const KSambaSubRecord &aRec, Bool_t bPrint) const
{
	//Compares two objects and their member variables to test for equality.
	//If bPrint is set to true, then a message for each member variable that is different
	//will print to standard out. Otherwise, this method will return false and quit
	//checking member variables as soon as it finds a unequal data member.
	
	Bool_t bIsEqual = true; //assume its true, then test for differences
	
	//call the base class's IsSame methods
	if(!this->KSubRecord::IsSame(aRec,bPrint)){
		bIsEqual = false;
		if(!bPrint)
			return false;  //if we're not printing out, just return false at first failure
		//the operator== method uses this functionality.
	}
	
	if(fSambaEventNumber != aRec.fSambaEventNumber){
		bIsEqual = false;
		if (bPrint) 
			cout << "KSambaSubRecord fSambaEventNumber Not Equal. lhs: " 
			<< fSambaEventNumber << " != rhs " << aRec.fSambaEventNumber << endl;	
		else
			return false;  
	}
	
	if(fSambaDAQNumber != aRec.fSambaDAQNumber){
		bIsEqual = false;
		if (bPrint) 
			cout << "KSambaSubRecord fSambaDAQNumber Not Equal. lhs: " 
			<< fSambaDAQNumber << " != rhs " << aRec.fSambaDAQNumber << endl;		
		else
			return false;  
	}
	
	
	return bIsEqual;
}


void KSambaSubRecord::Compact(void)
{
	//make the event class as small as possible. this calls 'Compact' for all member
	//variables that are KDS classes, member variables that can be compacted (such as TBits)
	//and base classes
	
	KSubRecord::Compact();
}


