//_____________________________________________
//
// KSingleBoloSubRecord.cxx
// KDataStructure
//
// Author: Adam Cox <mailto:adam.cox@ik.fzk.de> on 3/25/10.
//
// * Copyright 2010 Karlsruhe Institute of Technology. All rights reserved.
//
// A base class for all SingleBolo Sub Records. The SingleBolo Sub records
// contain data from an individual bolometer. 
//

#include "KSingleBoloSubRecord.h"
#include <stdio.h>
#include <iostream>
#include <cstring> 

using namespace std;

ClassImp(KSingleBoloSubRecord);

KSingleBoloSubRecord::KSingleBoloSubRecord(void)
{
	InitializeMembers();
	
	//we must take care of members on the heap outside of 
	//InitializeMembers...
	
	
}

KSingleBoloSubRecord::KSingleBoloSubRecord(const KSingleBoloSubRecord &aRec)
: KSubRecord(aRec)
{
	CopyLocalMembers(aRec);

}

KSingleBoloSubRecord& KSingleBoloSubRecord::operator=(const KSingleBoloSubRecord &aRec)
{
	if(&aRec == this) {
		return *this;
		cout << "KSingleBoloSubRecord:operator= Woops - copying to yourself" << endl;
	}

	this->KSubRecord::operator=(aRec);
	CopyLocalMembers(aRec);
	
	
	return *this;
}

void KSingleBoloSubRecord::CopyLocalMembers(const KSingleBoloSubRecord &aRec)
{

	SetDetectorNumber(aRec.GetDetectorNumber());
	SetEnergyHeat(aRec.GetEnergyHeat());

}

KSingleBoloSubRecord::~KSingleBoloSubRecord(void)
{
	//Does calling clear at destruction take too much computing time?
  Clear("C");
	
	
}

void KSingleBoloSubRecord::Clear(Option_t *opt)
{
	//Clear the base classes and then clear/delete any local
  //members. Its necessary for this Clear method to exist
  //in the case that instances of this object are stored
  //inside of a TClonesArray
  //Also, if this class holds any TClonesArrays, it must call
  //TClonesArray::Clear("C")
	KSubRecord::Clear(opt);
	
  //Clear and delete local objects here. 
	
  //Re initialize local members here and prepare for the next use of this class.
  InitializeMembers();
	
}

void KSingleBoloSubRecord::InitializeMembers(void)
{
  //WARNING - THIS METHOD SHOULD NEVER ALLOCATE SPACE FOR POINTERS
  //ONLY SET MEMBERS ON THE STACK TO THEIR INITIAL VALUES
	SetDetectorNumber(-99);
	SetEnergyHeat(-99.0);
}


Bool_t KSingleBoloSubRecord::IsSame(const KSingleBoloSubRecord &aRec, Bool_t bPrint) const
{
	//Compares two objects and their member variables to test for equality.
	//If bPrint is set to true, then a message for each member variable that is different
	//will print to standard out. Otherwise, this method will return false and quit
	//checking member variables as soon as it finds a unequal data member.
	
	Bool_t bIsEqual = true; //assume its true, then test for differences
	
	//call the base class's IsSame methods
	if(!this->KSubRecord::IsSame(aRec,bPrint)){
		bIsEqual = false;
		if(!bPrint)
			return false;  //if we're not printing out, just return false at first failure
		//the operator== method uses this functionality.
	}
	
	if(fDetectorNumber != aRec.fDetectorNumber){
		if (bPrint) 
			cout << "KSingleBoloSubRecord fDetectorNumber Not Equal. lhs: " 
			<< fDetectorNumber << " != rhs " << aRec.fDetectorNumber << endl;		
		bIsEqual = false;
		if(!bPrint)
			return false;  //if we're not printing out, just return false at first failure
		//the operator== method uses this functionality.
	}
	
	if(fEnergyHeat != aRec.fEnergyHeat){
		bIsEqual = false;
		if (bPrint) 
			cout << "KSingleBoloSubRecord fEnergyHeat Not Equal. lhs: " 
			<< fEnergyHeat << " != rhs " << aRec.fEnergyHeat << endl;		
		else
			return false;  
	}
	
	return bIsEqual;
}


void KSingleBoloSubRecord::Compact(void)
{
	//make the event class as small as possible. this calls 'Compact' for all member
	//variables that are KDS classes, member variables that can be compacted (such as TBits)
	//and base classes
	
	KSubRecord::Compact();
}


