/*
 *  fillEventTest.cxx
 *  KDataStructure
 *
 *  Created by Adam Cox on 3/18/10.
 *  Copyright 2010 Karlsruhe Institute of Technology. All rights reserved.
 *
 */
#include "KHLAEvent.h"
#include "KSingleBoloSubRecord.h"
#include "KSambaSubRecord.h"
#include "KDataReader.h"
#include "KDataWriter.h"
#include "TProcessID.h"

#include <iostream>

using namespace std;

void makeInputFile1(Int_t &counter);
void makeInputFile2(Int_t &counter);
void RunTest(const char* file, bool opt);


int main(int , char** ) 
{
	cout << "fillEventTest" << endl;
	
	
	//THIS FIRST PART JUST CREATES TWO ROOT FILES TO USE FOR 
	//DEMONSTRATING THE BUG LATER. 
	Int_t j = 0;
	makeInputFile1(j);
	makeInputFile2(j);
	
	cout << "OKAY - INPUT FILES HAVE BEEN CREATED. NOW I MAKE TWO TESTS TO COPY EVENTS" << endl;
	cout << "FROM THESE TWO FILES INTO A THIRD FILE. In the first attempt (test1)," << endl;
	cout << "I copy events from input1.root and input2.root into the file without setting" << endl;
	cout << "the TRef in the KHLAEvent::operator= method. In test2, I set a flag so that" << endl;
	cout << "KHLAEvent::operator= method behaves differently. I expect them to be exactly the" << endl;
	cout << "same, but they are NOT. Can you tell me why and how to fix this?" << endl;
	
	cout << "run test1" << endl;
	RunTest("test1.root",true);
	
	cout << "run test2" << endl;
	cout << "now I do the same thing, but this time use the fAssignmenOptionNoTref == false setting in the KHLAEvent::operator= method" << endl;
	cout << endl;
	
	RunTest("test2.root",false);
	
	return 1;
	
}



void makeInputFile1(Int_t &j)
{
	
	cout << "Create input1.root" << endl;
	//create file 1. print out via Scan.
	KDataWriter f1("input1.root");  //this class does all the branch setting and stuff for me
	KHLAEvent *e1 = (KHLAEvent *)f1.GetEvent();
	
	for(Int_t i = 0; i < 8; i++)
	{
		Int_t ObjectNumber = TProcessID::GetObjectCount();
		
		KSingleBoloSubRecord *bolo = e1->AddBolo();
		bolo->SetEnergyHeat(10*i+1);
		bolo->SetDetectorNumber(1);
		e1->SetStamp(j++);
		
		KSambaSubRecord *samba = e1->AddSamba();
		samba->SetSambaEventNumber(i);
		samba->SetSambaDAQNumber(7);
		
		bolo->SetSambaRecord(samba);  
		
		f1.Fill();
		
		TProcessID::SetObjectCount(ObjectNumber);

		e1->Clear("C");
	}
	f1.Write();
	f1.Close();
	cout << endl;
	
}

void makeInputFile2(Int_t &j)
{
	cout << "Create input2.root" << endl;
	//create file 2. print out via Scan.
	KDataWriter f2("input2.root");  //this class does all the branch setting and stuff for me
	KHLAEvent *e2 = (KHLAEvent *)f2.GetEvent();
	
	for(Int_t i = 0; i < 8; i++)
	{
		Int_t ObjectNumber = TProcessID::GetObjectCount();
		
		KSingleBoloSubRecord *bolo = e2->AddBolo();
		bolo->SetEnergyHeat(10*i+201);
		bolo->SetDetectorNumber(2);
		e2->SetStamp(j++);
		
		KSambaSubRecord *samba = e2->AddSamba();
		samba->SetSambaEventNumber(i);
		samba->SetSambaDAQNumber(17);
		
		bolo->SetSambaRecord(samba); 
		
		f2.Fill();
		TProcessID::SetObjectCount(ObjectNumber);
		
		e2->Clear("C");
	}
	f2.Write();
	f2.Close();
}

void RunTest(const char *file, bool option)
{

	cout << endl;
	
	//create file 3, add events from file 1 and 2. 
	//do NOT use TRefs. print out via Scan.
	//THIS SEEMS TO WORK FINE BECAUSE I TURN OFF TRYING TO SET THE TREF
	//IN THE KHLAEVENT::OPERATOR= METHOD.
	//
	KDataWriter *f3 = new KDataWriter(file);  //this class does all the branch setting and stuff for me
	KHLAEvent *e3 = (KHLAEvent *)f3->GetEvent();
	
	KDataReader *fr1 = new KDataReader("input1.root");  
	KDataReader *fr2 = new KDataReader("input2.root");
	KHLAEvent *e1 = (KHLAEvent *)fr1->GetEvent();
	KHLAEvent *e2 = (KHLAEvent *)fr2->GetEvent();
	
	TTree *t1 = fr1->GetTTree();
	cout << "TTree::Scan for input1.root" << endl;
	t1->Scan("fEnergyHeat:fDetectorNumber:fSambaEventNumber:fSambaDAQNumber","","colsize = 30");
	cout << endl;
	
	TTree *t2 = fr2->GetTTree();
	cout << "TTree::Scan for input2.root" << endl;
	t2->Scan("fEnergyHeat:fDetectorNumber:fSambaEventNumber:fSambaDAQNumber","","colsize = 30");
	cout << endl;
	
	e3->SetAssignmentOptionNoTref(option);  
	
	e3->Clear("C");
	cout << "starting to fill" << endl;

	
	for(Int_t i = 0; i < 3; i++){
		
		//comment these next two lines out to see the difference. These lines are
		//required for the assignment operator to work properly!
		//fr1->GetTFile()->cd();
		//fr1->GetEntry(fr1->GetCurrentEntryNumber());
		
		*e3 = *e1;  //SEE THE ASSIGNMENT OPERATOR. 
		f3->Fill();
		e3->Clear("C");
		fr1->GetNextEntry();	
	}
	
	//comment these next two lines out to see the difference. These lines are
	//required for the assignment operator to work properly!
	//fr2->GetTFile()->cd();
	//fr2->GetEntry(fr2->GetCurrentEntryNumber());
	
	*e3 = *e2; 
	f3->Fill();
	fr2->GetNextEntry();
	e3->Clear("C");
	
	
	
	for(Int_t i = 0; i < 3; i++){
		//comment these next two lines out to see the difference. These lines are
		//required for the assignment operator to work properly!
		//fr1->GetTFile()->cd();
		//fr1->GetEntry(fr1->GetCurrentEntryNumber());
		
		*e3 = *e1;
		f3->Fill();
		fr1->GetNextEntry();
		e3->Clear("C");
	}
	
	
	for(Int_t i = 0; i < 3; i++){
		//comment these next two lines out to see the difference. These lines are
		//required for the assignment operator to work properly!
		//fr2->GetTFile()->cd();
		//fr2->GetEntry(fr2->GetCurrentEntryNumber());
		
		*e3 = *e2;
		f3->Fill();
		fr2->GetNextEntry();
		e3->Clear("C");
	}
	
	f3->Write();
	f3->Close();
	fr1->Close();
	fr2->Close();

	

	KDataReader *fr3 = new KDataReader(file);
	TTree *t3 = fr3->GetTTree();
	cout << "TTree::Scan for " << file << endl;
	cout << "Should be in order first 3 events from input1, one event from input2, next 3 events from input1, next 3 events from input2" << endl;
	t3->Scan("fEnergyHeat:fDetectorNumber:fSambaEventNumber:fSambaDAQNumber","","colsize = 30");
	cout << endl;
	
}
